import rasterio
import numpy as np

from rio_color.workers import atmos_worker, color_worker


def test_atmos():
    i = 77
    args = {"atmo": 0.03, "contrast": 15, "bias": 0.5, "out_dtype": "uint8"}

    with rasterio.open("tests/rgb8.tif") as src:
        ij, window = list(src.block_windows())[i]
        arr = atmos_worker([src], window, ij, args)
        assert arr.dtype == args["out_dtype"]
        assert arr.shape == (3, 32, 32)
        assert arr.max() <= 255
        max_uint8 = arr.max()

    with rasterio.open("tests/rgba8.tif") as src:
        ij, window = list(src.block_windows())[i]
        arr2 = atmos_worker([src], window, ij, args)
        # operates on rgb
        assert np.allclose(arr, arr2[0:3])
        # retains alpha band
        assert np.allclose(src.read(4, window=window), arr2[3])

    with rasterio.open("tests/rgb16.tif") as src:
        ij, window = list(src.block_windows())[i]
        arr = atmos_worker([src], window, ij, args)
        assert arr.dtype == args["out_dtype"]
        assert arr.shape == (3, 32, 32)
        assert arr.max() <= 255

    with rasterio.open("tests/rgb8.tif") as src:
        ij, window = list(src.block_windows())[i]
        args["out_dtype"] = "uint16"
        arr = atmos_worker([src], window, ij, args)
        assert arr.dtype == args["out_dtype"]
        assert arr.shape == (3, 32, 32)
        assert arr.max() > max_uint8


def test_color():
    i = 77
    args = {"ops_string": "gamma 3 0.95 gamma 1,2 0.99", "out_dtype": "uint8"}

    with rasterio.open("tests/rgb8.tif") as src:
        ij, window = list(src.block_windows())[i]
        arr = color_worker([src], window, ij, args)
        assert arr.dtype == args["out_dtype"]
        assert arr.shape == (3, 32, 32)
        assert arr.max() <= 255
        assert arr.min() >= 0
        max_uint8 = arr.max()

    with rasterio.open("tests/rgb16.tif") as src:
        ij, window = list(src.block_windows())[i]
        arr = color_worker([src], window, ij, args)
        assert arr.dtype == args["out_dtype"]
        assert arr.shape == (3, 32, 32)
        assert arr.max() <= 255
        assert arr.min() >= 0

    with rasterio.open("tests/rgb8.tif") as src:
        ij, window = list(src.block_windows())[i]
        args["out_dtype"] = "uint16"
        arr = color_worker([src], window, ij, args)
        assert arr.dtype == args["out_dtype"]
        assert arr.shape == (3, 32, 32)
        assert arr.max() <= 65535
        assert arr.max() > max_uint8
        assert arr.min() >= 0



import numpy as np
import pytest

from rio_color.utils import to_math_type, math_type, scale_dtype, magick_to_rio


@pytest.fixture
def arr():
    return (
        np.array(
            [[[1, 2], [3, 4]], [[5, 6], [7, 8]], [[9, 10], [11, 12]], [[0, 0], [0, 0]]]
        ).astype("uint8")
        * 10
    )


def test_to_math_type(arr):
    x = to_math_type(arr)
    assert x.dtype == math_type
    assert x.max() <= 1.0
    assert x.min() >= 0.0


def test_scale_dtype():
    arr = np.array([0.0, 1.0]).astype(math_type)
    x = scale_dtype(arr, "uint8")
    assert x.max() == 255
    assert x.min() == 0
    x = scale_dtype(arr, "uint16")
    assert x.max() == 65535
    assert x.min() == 0


def test_scale_round_trip(arr):
    x = to_math_type(arr)
    y = scale_dtype(x, arr.dtype)
    assert np.array_equal(arr, y)


def test_magick_to_rio():
    ops = magick_to_rio(
        "-channel B -sigmoidal-contrast 4 -gamma 0.95 "
        "-channel r -gamma 1.10 "
        "-channel rgb -sigmoidal-contrast 1x55% "
        "-channel G -gamma 0.9 "
        "-modulate 100,125 "
        "+channel -sigmoidal-contrast 3,40% "
        "-modulate 222,135 "
    )

    assert ops == " ".join(
        [
            "sigmoidal B 4 0.5",
            "gamma B 0.95",
            "gamma R 1.10",
            "sigmoidal RGB 1 0.55",
            "gamma G 0.9",
            "saturation 1.25",
            "sigmoidal RGB 3 0.4",
            "saturation 1.35",
        ]
    )



import os

from click import UsageError
from click.testing import CliRunner
import numpy as np
import pytest
import rasterio
from rasterio.enums import Compression

from rio_color.scripts.cli import color, atmos, check_jobs


def equal(r1, r2):
    with rasterio.open(r1) as src1:
        with rasterio.open(r2) as src2:
            return np.array_equal(src1.read(), src2.read())


def test_atmos_cli(tmpdir):
    output = str(tmpdir.join("atmosj1.tif"))
    runner = CliRunner()
    result = runner.invoke(
        atmos,
        ["-a", "0.03", "-b", "0.5", "-c", "15", "-j", "1", "tests/rgb8.tif", output],
    )
    assert result.exit_code == 0
    assert os.path.exists(output)

    output2 = str(tmpdir.join("atmosj2.tif"))
    runner = CliRunner()
    result = runner.invoke(
        atmos,
        ["-a", "0.03", "-b", "0.5", "-c", "15", "-j", "2", "tests/rgb8.tif", output2],
    )
    assert result.exit_code == 0
    assert os.path.exists(output2)

    assert equal(output, output2)


def test_color_cli(tmpdir):
    output = str(tmpdir.join("colorj1.tif"))
    runner = CliRunner()
    result = runner.invoke(
        color,
        [
            "-d",
            "uint8",
            "-j",
            "1",
            "tests/rgb8.tif",
            output,
            "gamma 3 1.85",
            "gamma 1,2 1.95",
            "sigmoidal 1,2,3 35 0.13",
            "saturation 1.15",
        ],
    )
    assert result.exit_code == 0
    assert os.path.exists(output)

    output2 = str(tmpdir.join("colorj2.tif"))
    result = runner.invoke(
        color,
        [
            "-d",
            "uint8",
            "-j",
            "2",
            "tests/rgb8.tif",
            output2,
            "gamma 3 1.85",
            "gamma 1,2 1.95",
            "sigmoidal 1,2,3 35 0.13",
            "saturation 1.15",
        ],
    )
    assert result.exit_code == 0
    assert os.path.exists(output2)

    assert equal(output, output2)


def test_bad_op(tmpdir):
    output = str(tmpdir.join("noop.tif"))
    runner = CliRunner()
    result = runner.invoke(
        color, ["-d", "uint8", "-j", "1", "tests/rgb8.tif", output, "foob 115"]
    )
    assert result.exit_code == 2
    assert "foob is not a valid operation" in result.output
    assert not os.path.exists(output)


def test_color_jobsn1(tmpdir):
    output = str(tmpdir.join("colorj1.tif"))
    runner = CliRunner()
    result = runner.invoke(
        color,
        [
            "-d",
            "uint8",
            "-j",
            "-1",
            "tests/rgb8.tif",
            output,
            "gamma 1,2,3 1.85 sigmoidal rgb 35 0.13",
        ],
    )
    assert result.exit_code == 0
    assert os.path.exists(output)


def test_check_jobs():
    assert 1 == check_jobs(1)
    assert check_jobs(-1) > 0
    with pytest.raises(UsageError):
        check_jobs(0)


def test_creation_opts(tmpdir):
    output = str(tmpdir.join("color_opts.tif"))
    runner = CliRunner()
    result = runner.invoke(
        color,
        [
            "--co",
            "compress=jpeg",
            "tests/rgb8.tif",
            output,
            "gamma 1,2,3 1.85 sigmoidal rgb 35 0.13",
        ],
    )
    assert result.exit_code == 0

    with rasterio.open(output, "r") as src:
        assert src.compression == Compression.jpeg

    output = str(tmpdir.join("color_opts.tif"))
    runner = CliRunner()
    result = runner.invoke(
        color, ["--co", "compress=jpeg", "tests/rgb8.tif", output, "gamma 1,2,3 1.85"]
    )
    assert result.exit_code == 0
    with rasterio.open(output, "r") as src:
        assert src.compression == Compression.jpeg

    output = str(tmpdir.join("atmos_opts.tif"))
    runner = CliRunner()
    result = runner.invoke(
        atmos,
        [
            "--co",
            "compress=jpeg",
            "-a",
            "0.03",
            "-b",
            "0.5",
            "-c",
            "15",
            "-j",
            "1",
            "tests/rgb8.tif",
            output,
        ],
    )
    assert result.exit_code == 0
    with rasterio.open(output, "r") as src:
        assert src.compression == Compression.jpeg


def test_color_cli_rgba(tmpdir):
    output = str(tmpdir.join("colorj1.tif"))
    runner = CliRunner()
    result = runner.invoke(
        color,
        [
            "-d",
            "uint8",
            "-j",
            "1",
            "tests/rgba8.tif",
            output,
            "gamma 3 1.85",
            "gamma 1,2 1.95",
            "sigmoidal 1,2,3 35 0.13",
            "saturation 1.15",
        ],
    )
    assert result.exit_code == 0

    with rasterio.open("tests/rgba8.tif") as src:
        with rasterio.open(output) as out:
            assert out.profile["count"] == 4
            # Alpha band is unaltered
            assert np.array_equal(src.read(4), out.read(4))


def test_color_cli_16bit_photointerp(tmpdir):
    output = str(tmpdir.join("color16color.tif"))
    runner = CliRunner()
    result = runner.invoke(
        color,
        [
            "-d",
            "uint16",
            "-j",
            "1",
            "tests/rgb16.tif",
            output,
            "gamma 3 1.85",
            "gamma 1,2 1.95",
        ],
    )
    assert result.exit_code == 0

    with rasterio.open("tests/rgb16.tif") as src:
        with rasterio.open(output) as out:
            assert out.colorinterp == src.colorinterp


def test_color_empty_operations(tmpdir):
    output = str(tmpdir.join("color.tif"))
    runner = CliRunner()
    result = runner.invoke(color, ["tests/rgb8.tif", output])
    assert result.exit_code == 2
    assert not os.path.exists(output)

    result = runner.invoke(color, ["tests/rgb8.tif", output, ", , ,"])
    assert result.exit_code == 2


def test_as_color(tmpdir):
    runner = CliRunner()
    result = runner.invoke(atmos, ["-a", "0.03", "--as-color", "foo.tif", "bar.tif"])
    assert result.exit_code == 0
    assert not os.path.exists("bar.tif")
    assert (
        result.output.strip()
        == "rio color foo.tif bar.tif gamma g 0.99, gamma b 0.97, sigmoidal rgb 10.0 0.15"
    )
